#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import imp
import json
import mock
import os
import unittest

from ci_utils import data_store

from ci_libs import utils
from ci_libs.image import ImageObjectNotFound
from tests import capture_stdout

import urllib2
urllib2.urlopen = mock.Mock()


class FakeUrlOpenReturn:
    """Object to mock the return of urllib2.urlopen."""

    def __init__(self, code, content):
        self.code = code
        self.content = content

    def __repr__(self):
        return self.content

    def __iter__(self):
        return (line for line in self.content.split("\n"))

    def read(self):
        return self.content


artifact_data = {
    "objects": [
        {
            "id": 6,
            "name": "d62d3d9a-ac3d-11e3-847d-fa163eaf5928.img",
            "reference": "http://www.example.com",
            "resource_uri": "/api/v1/ticketartifact/6/",
            "ticket": {
                "id": 4,
            },
            "type": "IMAGE"
        },
    ]
}

ticket_data = {
    "objects": [
        {
            "id": 1,
        },
        {
            "id": 4,
        },
        {
            "id": 2,
        },
    ],
}


class TestGetTicketImage(unittest.TestCase):

    def setUp(self):
        super(TestGetTicketImage, self).setUp()
        self.cli = imp.load_source(
            "ubuntu-ci",
            os.path.join(os.path.dirname(__file__), "../ubuntu-ci")
        )
        self.image_path = os.path.join(
            utils.HOME, 'd62d3d9a-ac3d-11e3-847d-fa163eaf5928.img')
        self.addCleanup(self._remove_image)

    def _remove_image(self):
        if os.path.exists(self.image_path):
            os.remove(self.image_path)

    @mock.patch('ci_utils.data_store.create_for_ticket')
    def test_get_ticket_image(self, mock_create_datastore):
        urllib2.urlopen.side_effect = [
            FakeUrlOpenReturn(200, json.dumps(artifact_data))]
        mock_datastore = mock.Mock()
        mock_datastore.get_file.return_value = buffer(" ")
        mock_create_datastore.return_value = mock_datastore

        args = self.cli.parse_arguments(['get_image', '-t', '4', '-n',
                                         self.image_path])
        with capture_stdout(args.func, args) as cm:
            self.assertEqual(
                cm,
                "Successfully downloaded image for ticket #4.\n"
                "Image file can be found at: {}\n".format(self.image_path))
        mock_datastore.assert_called_once()

    @mock.patch('ci_utils.data_store.create_for_ticket')
    def test_get_ticket_image_not_found(self, mock_create_datastore):
        urllib2.urlopen.side_effect = [
            FakeUrlOpenReturn(200, json.dumps(artifact_data))]
        mock_datastore = mock.Mock()
        mock_datastore.get_file.side_effect = data_store.DataStoreException(
            "Failed to get file: d62d3d9a-ac3d-11e3-847d-fa163eaf5928.img, "
            " Error: Object GET failed: https://swift.canonistack.canonical."
            "com:443/v1/AUTH_2de6e093b42c44b58a09758c347535ff/ticket-1-image/"
            "d62d3d9a-ac3d-11e3-847d-fa163eaf5928.img 404 Not Found")
        mock_create_datastore.return_value = mock_datastore

        args = self.cli.parse_arguments(['get_image', '-t', '4', '-n',
                                         self.image_path])
        with self.assertRaises(ImageObjectNotFound):
            args.func(args)

    @mock.patch('ci_utils.data_store.DataStore')
    def test_get_ticket_image_already_exists(self, mock_data_store):
        urllib2.urlopen.side_effect = [
            FakeUrlOpenReturn(200, json.dumps(artifact_data))]
        open(self.image_path, 'a').close()
        args = self.cli.parse_arguments(['get_image', '-t', '4', '-n',
                                         self.image_path])
        with capture_stdout(args.func, args) as cm:
            self.assertEqual(
                "{} already exists.\n".format(self.image_path),
                cm)

    @mock.patch('ci_utils.data_store.create_for_ticket')
    def test_get_last_completed_ticket_image(self, mock_create_datastore):
        urllib2.urlopen.side_effect = [
            FakeUrlOpenReturn(200, json.dumps(ticket_data)),
            FakeUrlOpenReturn(200, json.dumps(artifact_data))
        ]
        mock_datastore = mock.Mock()
        mock_datastore.get_file.return_value = buffer(" ")
        mock_create_datastore.return_value = mock_datastore

        args = self.cli.parse_arguments(['get_image', '-n', self.image_path])
        with capture_stdout(args.func, args) as cm:
            self.assertEqual(
                cm,
                "Successfully downloaded latest release candidate image.\n"
                "Image file can be found at: {}\n".format(self.image_path))
        mock_datastore.assert_called_once()

    def test_get_last_completed_ticket_none_complete(self):
        data = {'objects': []}
        urllib2.urlopen.side_effect = [
            FakeUrlOpenReturn(200, json.dumps(data))]
        args = self.cli.parse_arguments(['get_image', '-n', self.image_path])
        with self.assertRaises(SystemExit) as cm:
            args.func(args)
        self.assertEqual("No completed tickets found.", cm.exception.message)
