#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils import (
    amqp_utils,
    amqp_worker,
)
import tstrun
from tstrun import testbed


class TestRunnerWorker(amqp_worker.AMQPWorker):

    def __init__(self):
        super(TestRunnerWorker, self).__init__('image_test')
        self.data_store = None

    def save_artifact(self, logger,  results, name, value, description=None,
                      kind='LOGS'):
        """Save an artifact catching and reporting exceptions.

        This should only be used to upload artifacts after the pass/fail status
        has been established to it's safe to fail the upload.

        :param logger: To report execution.

        :param results: The dict holding the 'artifacts' attribute.

        :param name: The artifact name.

        :param value: The artifact value as a string.

        :param description: For reporting purposes.

        :param kind: The kind of artifact (defaults to 'LOGS').
        """
        if description is None:
            description = name
        try:
            logger.info('Saving {}'.format(description))
            url = self.data_store.put_file(name, value, 'text/plain')
            self._create_artifact('{}.{}'.format(self.logger_name, name),
                                  url, results, kind)
        except:
            logger.exception('Unable to upload {}'.format(description))

    def save_testbed_console(self, logger, results, test_bed):
        self.save_artifact(logger, results,
                           'testbed-cloud-init.log',
                           test_bed.get_cloud_init_console(),
                           'testbed console')

    def save_testbed_artifacts(self, logger, results, test_bed, package):
        subunit_path = '{}.subunit'.format(package)
        self.save_artifact(
            logger, results,
            subunit_path, test_bed.get_remote_content(subunit_path),
            'subunit results for {}'.format(package),
            'RESULTS')
        package_log_path = '{}.log'.format(package)
        self.save_artifact(
            logger, results,
            package_log_path, test_bed.get_remote_content(package_log_path),
            'adt-run log for {}'.format(package))
        summary_log_path = '{}-summary.log'.format(package)
        self.save_artifact(
            logger, results,
            summary_log_path, test_bed.get_remote_content(summary_log_path),
            'adt-run summary for {}'.format(package))

    def handle_request(self, logger, params):
        ticket_id = params['ticket_id']
        progress_queue = params['progress_trigger']
        image_id = params['image_id']
        package_list = params['package_list']

        results = {}

        def status_cb(msg):
            logger.info(msg)
            amqp_utils.progress_update(progress_queue, {'message': msg})

        self.data_store = self._create_data_store(ticket_id)
        try:
            status_cb('Setting up the testbed for ticket {}'.format(ticket_id))
            flavors = tstrun.get_auth_config()['tr_flavors']
            test_bed = testbed.TestBed('testbed-{}'.format(progress_queue),
                                       flavors, image_id, status_cb)
        except:
            logger.exception(
                'The testbed creation for {} failed'.format(ticket_id))
            return amqp_utils.progress_failed, results

        try:
            test_bed.setup()
        except:
            logger.exception(
                'The testbed setup for {} failed'.format(ticket_id))
            if test_bed.instance is not None:
                self.save_testbed_console(logger, results, test_bed)
                test_bed.teardown()
            return amqp_utils.progress_failed, results

        self.save_testbed_console(logger, results, test_bed)
        status_cb('The test bed is ready')
        # The tests will succeed unless they fail ;)
        notify = amqp_utils.progress_completed
        try:
            for package in package_list:
                if params.get('cancelled'):
                    logger.error('The request for {} has been cancelled,'
                                 ' exiting'.format(ticket_id))
                    return amqp_utils.progress_failed, results
                status_cb('Testing {}'.format(package))
                # uci-vms shell adt-run ...  --- adt-virt-null
                return_code = test_bed.run_test(package)
                # 0 is success, 8 is skipped and considered a success
                if return_code not in (0, 8):
                    # At least one test failed
                    notify = amqp_utils.progress_failed
                self.save_testbed_artifacts(logger, results, test_bed, package)
            status_cb('Test completed for ticket {}'.format(ticket_id))
            return notify, results
        except:
            logger.exception(
                'Exception while handling ticket {}'.format(ticket_id))
            raise
        finally:
            test_bed.teardown()


if __name__ == '__main__':
    TestRunnerWorker().main(tstrun.internal_rabbit_queue)
